@@ -55,6 +55,7 @@ from __future__ import absolute_import, division, print_function
 
 import os
 import json
+import uuid
 import types
 import logging
 from io import BytesIO
@@ -70,7 +71,7 @@ from matplotlib import cm
 from glue.external import six
 from glue import core
 from glue.core.data import Data
-from glue.core.component_id import ComponentID
+from glue.core.component_id import ComponentID, PixelComponentID
 from glue.core.component import (Component, CategoricalComponent,
                                  DerivedComponent, CoordinateComponent)
 from glue.core.subset import (OPSYM, SYMOP, CompositeSubsetState,
@@ -370,12 +371,13 @@ class GlueSerializer(object):
 
     def dumps(self, indent=None):
         result = self.dumpo()
-        return json.dumps(result, indent=indent, default=self.json_default)
+        return json.dumps(result, default=self.json_default,
+                          indent=indent, sort_keys=True)
 
     def dump(self, outfile, indent=None):
         result = self.dumpo()
         return json.dump(result, outfile, default=self.json_default,
-                         indent=indent)
+                         indent=indent, sort_keys=True)
 
 
 class GlueUnSerializer(object):
@@ -478,12 +480,14 @@ loader = GlueUnSerializer.unserializes
 
 @saver(dict)
 def _save_dict(state, context):
-    return dict(contents=json.dumps(state))
+    return dict(contents=dict((context.id(key), context.id(value))
+                for key, value in state.items()))
 
 
 @loader(dict)
 def _load_dict(rec, context):
-    return json.loads(rec['contents'])
+    return dict((context.object(key), context.object(value))
+                for key, value in rec['contents'].items())
 
 
 @saver(CompositeSubsetState)
@@ -666,9 +670,7 @@ def _load_data(rec, context):
 
     comps = [list(map(context.object, [cid, comp]))
              for cid, comp in rec['components']]
-    comps = sorted(comps,
-                   key=lambda x: isinstance(x[1], (DerivedComponent,
-                                                   CoordinateComponent)))
+
     for cid, comp in comps:
         if isinstance(comp, CoordinateComponent):
             comp._data = result
@@ -733,7 +735,10 @@ def _load_data_4(rec, context):
         return tuple(context.object(cid) for cid in cids)
     result._key_joins = dict((context.object(k), (load_cid_tuple(v0), load_cid_tuple(v1)))
                              for k, v0, v1 in rec['_key_joins'])
-    result.uuid = rec['uuid']
+    if 'uuid' in rec and rec['uuid'] is not None:
+        result.uuid = rec['uuid']
+    else:
+        result.uuid = str(uuid.uuid4())
 
 
 @saver(ComponentID)
@@ -746,6 +751,20 @@ def _load_component_id(rec, context):
     return ComponentID(rec['label'], rec['hidden'])
 
 
+@saver(PixelComponentID)
+def _save_component_id(cid, context):
+    return dict(axis=cid.axis, label=cid.label, hidden=cid.hidden)
+
+
+@loader(PixelComponentID)
+def _load_component_id(rec, context):
+    if 'axis' in rec:
+        axis = rec['axis']
+    else:  # backward-compatibility
+        axis = int(rec['label'].split()[2])
+    return PixelComponentID(axis, rec['label'], rec['hidden'])
+
+
 @saver(Component)
 def _save_component(component, context):
 
